import os
import shutil
import sys

import jsonpickle


def capitalize_first_letter(identifier):
    return identifier[0].capitalize() + identifier[1:]


def create_directory(directory, force):
    if os.path.exists(directory):
        if force:
            shutil.rmtree(directory, ignore_errors=True)
        else:
            exit_with_error("output directory {} already exists", directory)

    os.makedirs(directory)


def to_underscore_separated_name(name):
    result = ""
    first_char = True
    for ch in name:
        if ch.islower():
            next_char = ch
        else:
            next_char = ch.lower()
            if not first_char:
                next_char = "_" + next_char
        first_char = False

        result += next_char
    return result


def class_name_to_private_module_name(class_name):
    # The leading underscore indicates that users are not intended to import
    # the class module individually.
    return "_" + to_underscore_separated_name(class_name)


def unpickle_file(path):
    with open(path, mode="rt") as file_obj:
        contents = file_obj.read()
        return jsonpickle.decode(contents)


def exit_with_error(message, *args):
    sys.stderr.write("error : " + message.format(*args) + "\n")
    sys.exit(1)



from __future__ import absolute_import

import os
import sys

if not __package__:
    path = os.path.dirname(os.path.dirname(__file__))
    sys.path.insert(0, path)

from jschema_to_python.driver import main as _main  # noqa

if __name__ == "__main__":
    sys.exit(_main())



import sys
from jschema_to_python.python_file_generator import PythonFileGenerator
import jschema_to_python.utilities as util


class InitFileGenerator(PythonFileGenerator):
    def __init__(self, module_name, root_schema, root_class_name, output_directory):
        super(InitFileGenerator, self).__init__(output_directory)
        self.module_name = module_name
        self.root_schema = root_schema
        self.root_class_name = root_class_name

    def __del__(self):
        sys.stdout = sys.__stdout__

    def generate(self):
        file_path = self.make_output_file_path("__init__.py")
        with open(file_path, "w") as sys.stdout:
            self.write_generation_comment()
            self.write_import_statements()

    def write_import_statements(self):
        self.write_import_statement(self.root_class_name)

        definition_schemas = self.root_schema.get("definitions")
        if definition_schemas:
            definition_keys = sorted(definition_schemas.keys())
            for definition_key in definition_keys:
                class_name = util.capitalize_first_letter(definition_key)
                self.write_import_statement(class_name)

    def write_import_statement(self, class_name):
        class_module_name = util.class_name_to_private_module_name(class_name)
        print(
            "from "
            + self.module_name
            + "."
            + class_module_name
            + " import "
            + class_name
        )



import os

from jschema_to_python import __version__


class PythonFileGenerator(object):
    def __init__(self, output_directory):
        self.output_directory = output_directory

    def write_generation_comment(self):
        print(
            "# This file was generated by "
            + __package__
            + " version "
            + __version__
            + ".\n"
        )

    def make_output_file_path(self, file_name):
        return os.path.join(self.output_directory, file_name)
